function [SegmClass,TypClass]=NNclassification(d,PAT)
% EEG3classification.M - Self organizing neural network classification
% of signal d with segments features in matrix PAT
% Input values
%   d	- given signal (column vector)
%   PAT - matrix (m,n) containing information for segments of signal d
%	     PAT(:,1) - segment number (1..Q)
%	     PAT(:,2) - initial segment index
%	     PAT(:,3) - final segment index
%	     PAT(:,[4:n]) - submatrix of signal features with each its
%			    row standing for one segment
% Output values
%   SegmClass - segment indices and class association
%   TypClass  - class number and typical segment index
% Functions used: cl_disp.m  - axes position definition
%	       cl_separ.m - separation lines for two elements input vectors
%	       cl_contr.m - figure modification
%	       cl_impl.m, cl_expl.m - position change
%% Input values definition
   delete(get(0,'children'));
   if exist('d')==0
      load SignalFEATURE;
   end
 %%  Initialization
   d=d';
  [m,n]=size(PAT); signal=d;
  for i=1:m
    start=PAT(i,2); stop=PAT(i,3); segmentlength(i)=stop-start+1;
    x(1:segmentlength(i),i)=d(start:stop);
  end
  PATT=PAT(:,4:n)';  [R,Q]=size(PATT);
  S=input('Initial number of classes (=1): ');
  NoMaxClass=input('Final number of classes (=5): ');
  TrainCycle=input('Number of training cycles (=900): ');
  lrate=0.05; Rt=0.25; ExpTheta=-0.03; 
  NoWin=R*(R-1)/2; NoCWin=ceil(NoWin/3);
  ZobrFreq=1; AktFreq=0;
%%  Kohonen net initialization
  rand('seed',10);
  MaxPatt=max(PATT'); MinPatt=min(PATT');
    for i=1:S
      for j=1:R
	    wo(i,j)=rand*(MaxPatt(j)-MinPatt(j))+MinPatt(j);
      end
    end
%%  Mapping of each input with each output
  pal=['k' 'm' 'r' 'c' 'g' 'b' 'y' 'w']; penwidth=1;
  classname=['A' 'B' 'C' 'D' 'E' 'F' 'G'];
  [FIG,subfig]=cl_disp(R); figure(FIG); win=1;
  for j=1:R
    for i=1:R
      if i>j
	subplot(subfig(win));
	  xi=min(PATT(i,:)); xa=max(PATT(i,:)); dx=xa-xi;
	  yi=min(PATT(j,:)); ya=max(PATT(j,:)); dy=ya-yi;
	  set(subfig(win),'xlim',[xi-dx/10 xa+dx/10],...
			  'ylim',[yi-dy/10 ya+dy/10],'visible','on');
	  pop(win)=text('color','w','position',[dx/2 ya+dy/20 0],...
			'string',['(' num2str(j) ',' num2str(i) ')'],...
			'fontsize',10,'HorizontalAlignment','left');
	  for vst=1:Q
	     HInputText(win,vst) = text('color','w','position',...
		 [PATT(i,vst) PATT(j,vst) 0],'string',num2str(vst),...
		 'fontsize',8,'visible','on','HorizontalAlignment','center');
	  end
	  for pcol=1:S
	     Hweight(win,pcol) = line('color',pal(pcol),'linestyle','o',...
				     'erase','xor','xdata',wo(pcol,i),...
				     'ydata',wo(pcol,j),'markersize',20,'LineWidth',5);
	  end
	  win=win+1;
      end
    end
  end
  drawnow;
%% Kohonen training
   ex=0;
   for ii=1:TrainCycle
      ex=ex+1;
      theta=100*exp(ExpTheta*ex);	   % Activation value evaluation
% Random pattern choice and its distance related to weights evaluation
      q=fix(rand*Q)+1; distance=dist(wo,PATT(:,q)); A=compet(-distance);
		       winner = min(find(A == 1));
% Decision of the winning neuron activity with repect to the theta value
      if distance(winner)<theta, active=1; else active=0; end;
% The new output unit creation for the winning and inactive neuron using
% a chosen resemblance factor Rt
  if active==0
	if S==NoMaxClass, break, end;
	S=S+1; [r,s]=size(wo); wo(r+1,s)=0;
	  dW=0*wo; dW(r+1,:)=wo(winner,:)+Rt*(PATT(:,q)'-wo(winner,:));
	  wo=wo+dW; ex=0;
  end
% The weight modification for the winning and active neuron using
% a chosen learning rate lrate
  if active==1
   	dW=0*wo;  dW(winner,:)=lrate*(PATT(:,q)'-wo(winner,:));
    wo=wo+dW;
  end
% Learning process visualization
  AktFreq=AktFreq+1;
  if AktFreq==ZobrFreq
	win=1;
	for j=1:R
	  for i=1:R
	    if i>j
	      subplot(subfig(win));
    		set(FIG,'name',...
		    ['cycle: ' num2str(ii) ' winner: ' num2str(winner)]);
	      if active==0
	      	pcol=S;
    		Hweight(win,pcol) = line('color',pal(pcol),'linestyle','o',...
					'erase','xor','xdata',wo(pcol,i),...
					'ydata',wo(pcol,j),'markersize',20,'LineWidth',5);
	    	set(Hweight(win,pcol),'xdata',wo(pcol,i),'ydata',wo(pcol,j));
	      end
	      if active==1
		    set(Hweight(win,winner),'xdata',wo(winner,i),...
					  'ydata',wo(winner,j));
	      end; win=win+1;
	    end
	  end
	end; AktFreq=0;
  end
  end
%% Classification of each segment into a class
  b=[]; for i=1:Q
    	  A=compet(-dist(wo,PATT(:,i))); b=[b full(A)];
	    end
  [class d]=find(b==1);
  for i=1:S
    gh=PATT(:,class==i); tr=find(class==i);
    w=(ones(size(gh,2),1)*wo(i,:))';  [differ ind]=min(sum((w-gh).^2));
    if exist('tr')==1, least(i)=tr(ind); else, least(i)=10000; end
  end
  win=1; 
    for j=1:R
	   for i=1:R
	     if i>j
	       subplot(subfig(win));
           for vst=1:Q,
		     set(HInputText(win,vst),'color',pal(class(vst)));
           end; win=win+1;
	     end
	   end
	 end
% Class boundaries plot in case of two input elements
  if R==2, cl_separ(S,wo,FIG,subfig); end
%% Signal features histogram plot
  figure('name','SIGNAL HISTOGRAM ','units','normal','position',[0.4,0.65,0.6,0.25]);
   maxnum=-inf; MP=max(max(PATT));
  for i=1:S
    akt=length(find(class(:)==i));
    if akt>maxnum, maxnum=akt; end
  end
  for i=1:S            
    pos=[0.005+(i-1)/S 0.006 1/S-0.01 0.8];
      axes('position',pos,'box','on'); 
          set(gca,'xtick',[],'ytick',[])
      CX=find(class(:)==i);
      if exist('CX')==1
	    for sel=1:length(CX)  % Move of the closest pattern to the first position
          if least(i)==CX(sel)
	         CX([1 sel])=CX([sel 1]); break
          end
        end
          PCLASS=PATT(:,CX); A=compet(-dist(wo,PCLASS(:,1))); % Patterns in separate class
          TCLASS=wo(find(A==1),:); MSE1=sum(dist(TCLASS,PCLASS))/length(PCLASS(1,:));
          MSE(i)=round(MSE1*1000)/1000;
          title(['CLASS ',classname(i),': ',num2str(MSE(i))],'color',pal(i))
     	posl=PATT(:,CX(1)); 
        line([1:length(posl); 1:length(posl)],[zeros(1,length(posl)); posl'+0.01],...
            'linewidth',25,'color',pal(i))
        line([0.1 length(PATT(:,1))+0.5],[0 0],'linewidth',1,'color','k')
        axis([0.1 length(PATT(:,1))+0.5 -0.1 maxnum*(1.1*MP)]);
	    text(0.15,0.25,[num2str(CX(1))]); hold on
    	for k=2:length(CX)
          posl=PATT(:,CX(k));
          line([1:length(posl); 1:length(posl)],...
               [zeros(1,length(posl))+1.1*(k-1)*MP; posl'+1.1*(k-1)*MP+0.01],...
               'linewidth',25,'color',[0.7 0.7 0.7])
          line([0.1 length(PATT(:,1))+0.5],[1.1*(k-1)*MP 1.1*(k-1)*MP],'linewidth',1,'color','k')
	      text(0.15,1.1*(k-1)*MP+0.25,[num2str(CX(k))]);
    	end; hold off
     end
  end
  crit=mean(MSE); woDIST=dist(wo,wo'); crit=crit/mean2(woDIST)*100;
  disp(['Mean value of feature distance function over all classes: ',num2str(crit)]) 
%% Signal segments plot
  figure(FIG); maxnum=-inf;
	       MP=max(signal); MM=min(signal);
  for i=1:S
    akt=length(find(class(:)==i));
    if akt>maxnum, maxnum=akt; end
  end
  for i=1:S
     pos=[0.5+(0.5*(i-1))/S 0.006 0.5/S-0.01 0.7];
     CLX=find(class(:)==i);
     for sel=1:length(CLX);
       if least(i)==CLX(sel)
	  CLX([1 sel])=CLX([sel 1]); break
       end
     end
     ccx(i)=axes('position',pos,'box','on');
     if exist('CLX')==1
       posl=x(1:segmentlength(CLX(1)),CLX(1));
       h=plot(posl,pal(i)); set(h,'linewidth',penwidth); v=axis;
       txt(i,1)=text(0,posl(1),[num2str(CLX(1))]);
       set(txt(i,1),'buttondownfcn','cl_contr','UserData',...
				      [segmentlength(CLX(1)) CLX(1)]); hold on
       plot(0:max(segmentlength),...
		  zeros(1,max(segmentlength)+1)+MM+1.05*(MP-MM),'w:' );
       line([0, max(segmentlength)],[0.55*(MP-MM) 0.55*(MP-MM)])
       for k=2:length(CLX)
    	 posl=x(1:segmentlength(CLX(k)),CLX(k))+1.1*(k-1)*(MP-MM);
	     h=plot(posl,pal(i)); set(h,'linewidth',0.5); % penwidth);
    	 txt(i,k)=text(0,posl(1),num2str(CLX(k)));
	     set(txt(i,k),'buttondownfcn','cl_contr','UserData',...
				       [segmentlength(CLX(k)) CLX(k)]);
       end; hold off
     end
     axis([0, max(segmentlength), MM-maxnum*(0.05*(MP-MM)),...
                                  MM+maxnum*(1.1*(MP-MM))])
     set(gca,'xtick',[],'ytick',[])
     if i==1
         title(['(c)   CLASS ',classname(i)],'color',pal(i))
     else
         title(['CLASS ',classname(i)],'color',pal(i))
     end
  end
%% Given signal plot and its segments boundaries
  eegseq=axes('position',[0.01 0.8 0.98 0.15],'xtick',[],'ytick',[],'box','on');
  fs=128; change=[0 cumsum(segmentlength)];
  for k=1:m
       posl=x(1:segmentlength(k),k); hold on;
       h=plot([change(k)+1:change(k+1)]/fs,posl,pal(class(k))); set(h,'linewidth',0.5); 
  end; hold off
    v=[1/fs change(end)/fs min(signal) max(signal)]; axis(v);
    for i=2:length(change)
        l=line([change(i) change(i)]/fs,[v(3) v(4)]);
        set(l,'LineStyle',':');
        text((change(i)-30)/fs,v(3)-0.1*(v(4)-v(3)),num2str(i-1));
    end
   set(gca,'ytick',[]); title('(a) SIGNAL SEGMENTS CLASSIFICATION');
%% Size change
  ax=length(subfig); 
  for i=1:ax
    set(subfig(i),'buttondownfcn','cl_impl');
  end
%% Results
   SegmClass=[[1:length(class)]' class];
   TypClass=[(1:length(least))' least'];
